<?php

namespace Drupal\stripe_checkout\Controller;

use Drupal\Core\Entity\Entity;
use Drupal\Core\Url;
use Stripe\Charge;
use Drupal\Core\Controller\ControllerBase;
use Stripe\Checkout\Session;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\stripe_api\StripeApiService;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Session\AccountProxy;

/**
 * Class StripeCheckoutController.
 *
 * @package Drupal\stripe_checkout\Controller
 */
class StripeCheckoutController extends ControllerBase {

  /**
   * Drupal\stripe_api\StripeApiService definition.
   *
   * @var \Drupal\stripe_api\StripeApiService
   */
  protected $stripeApiStripeApi;
  protected $currentUser;

  /**
   * {@inheritdoc}
   */
  public function __construct(StripeApiService $stripe_api_stripe_api, AccountProxy $current_user) {
    $this->stripeApiStripeApi = $stripe_api_stripe_api;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('stripe_api.stripe_api'),
      $container->get('current_user')
    );
  }

  /**
   * @param \Symfony\Component\HttpFoundation\Request $request
   *
   * @return \Symfony\Component\HttpFoundation\Response
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Exception
   */
  public function createSession(Request $request): Response {
    $entity_type = $request->get('entity_type');
    $entity_id = $request->get('entity_id');
    $field_name = $request->get('field_name');
    $delta = $request->get('delta');

    $this->getLogger('stripe_checkout.logger')->info(t('Creating new Stripe session, submitted data: @data', [
      '@data' => $request->getContent(),
    ]));

    if ($entity_type === NULL || $entity_id  === NULL || $field_name  === NULL || $delta === NULL) {
      throw new \RuntimeException('Required data is missing!');
    }

    /** @var \Drupal\Core\Entity\EntityInterface $entity */
    $entity = \Drupal::entityTypeManager()->getStorage($entity_type)->load($entity_id);
    $price = $entity->get($field_name)->get($delta)->value;
    $field_settings = $entity->$field_name->getSettings();
    $currency = $field_settings['currency'];
    if ($entity->get('body') !== NULL) {
      $description = str_replace('&nbsp;', '', trim(strip_tags($entity->get('body')->value)));
      $description = str_replace(['<br>', '<br/>', '<br />'], "\n", $description);
      $description = strip_tags($description);
    }

    try {
      $session = Session::create([
        'payment_method_types' => ['card'],
        'line_items' => [
          [
            'quantity' => 1,
            'price_data' => [
              'currency' => $currency,
              'unit_amount' => $price * 100,
              'product_data' => [
                'name' => $entity->label(),
                'description' => $description ?? $entity->label(),
                // @todo image.
                'metadata' => [
                  'entity_id' => $entity_id,
                  'entity_type' => $entity_type,
                  'uid' => $this->currentUser->id(),
                ],
              ],
            ],
          ],
        ],
        'mode' => 'payment',
        'customer_email' => $this->currentUser->getEmail(),
        'metadata' => [
          'module' => 'stripe_checkout',
          'field_name' => $field_name,
          'entity_id' => $entity_id,
          'entity_type' => $entity_type,
          'uid' => $this->currentUser->id(),
        ],
        'success_url' => Url::fromRoute('entity.node.canonical', ['node' => $entity_id],
          ['absolute' => TRUE, 'query' => ['checkout' => 'success']])->toString(),
        'cancel_url' => Url::fromRoute('entity.node.canonical', ['node' => $entity_id],
          ['absolute' => TRUE, 'query' => ['checkout' => 'failure']])->toString(),
      ]);
    }
    catch (\Exception $exception) {
      return new Response($exception->getMessage(), Response::HTTP_BAD_REQUEST);
    }

    $response_content = [
      'session_id' => $session->id,
    ];

    return new Response(json_encode($response_content), Response::HTTP_ACCEPTED);
  }

}
